package cz.muni.fi.pb138.demo.dom4j;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.util.Iterator;
import java.util.List;

import org.dom4j.Document;
import org.dom4j.DocumentException;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.XPath;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.SAXReader;
import org.dom4j.io.XMLWriter;

/**
 * Simple demo to illustrate features of dom4j
 * 
 * based on program snippets from dom4j Cookbook
 *
 * @author Tomas Pitner, tomp@fi.muni.cz
 * @version Feb 6, 2006
 *
 */

public class Demo {

    /**
     * @param aFile to be read and parser
     * @return the parser document
     * @throws DocumentException if something bad occurs
     */
    public static Document parseWithSAX(File aFile) throws DocumentException {
        SAXReader xmlReader = new SAXReader();
        Document doc = xmlReader.read(aFile);
        return doc;
    }

    /**
     * Create a demo dom4j document from scratch
     */
    public static Document createDocument() {
        Document document = DocumentHelper.createDocument();
        
        Element books = document.addElement("books");

        Element book1 = books.addElement("book");

        book1.addElement("author").addAttribute("name", "tomp").addAttribute(
                "location", "Czech Republic").addText("Tomas Pitner");
        book1.addElement("title").addText("Java for Beginners");

        Element book2 = books.addElement("book");
        
        book2.addElement("author").addAttribute("name", "nwalsh").addAttribute(
                "location", "UK").addText("Norm Walsh");
        book2.addElement("title").addText("Docbook - The Definitive Guide");

        return document;
    }

    /**
     * @param doc Document to be serialized
     * @param out stream to serialize to 
     * @param aEncodingScheme output encoding to use
     * @throws Exception
     */
    public static void serializetoXML(Document doc, OutputStream out,
            String aEncodingScheme) throws Exception {
        OutputFormat outformat = OutputFormat.createPrettyPrint();
        outformat.setEncoding(aEncodingScheme);
        XMLWriter writer = new XMLWriter(out, outformat);
        writer.write(doc);
        writer.flush();
    }

    /**
     * @param doc Document to be browsed (searched thru)
     */
    public static void browseRootChildren(Document doc) {

        XPath xpathSelector = DocumentHelper
                .createXPath("/books/book/author");
        
        List results = xpathSelector.selectNodes(doc);
        for (Iterator iter = results.iterator(); iter.hasNext();) {
            Element element = (Element) iter.next();
            System.out.println("Found element name="+element.getName());
        }
        System.out.println("---");

        // Tomas Pitner's location
        String loc = doc.valueOf("//author[@name='tomp']/@location");
        System.out.println("Tomas Pitner's location=" + loc);

        // the second book's author real name
        String name = doc.valueOf("/books/book[2]/author/text()");
        System.out.println("The 2nd book's author real name="+name);

        // The Norm's book name
        String bname = doc.valueOf("/books/book[author/@name='nwalsh']/title");
        System.out.println("The Norm's book name="+bname);

        // Number of authors located in CZ
        Number count = doc.numberValueOf("count(//author[@location='Czech Republic'])");
        System.out.println("Number of authors located in CZ="+count);
        System.out.println("---");
    }

    public static void main(String[] args) throws Exception {

        // create a sample Document
        Document newDoc = createDocument();
        
        // browse it
        browseRootChildren(newDoc);

        // serialize (write) the tree to file
        serializetoXML(newDoc, new FileOutputStream("demo.xml"), "UTF-8");

        // read the file as Document
        Document docFromFile = parseWithSAX(new File("demo.xml"));

        // print to console
        serializetoXML(docFromFile, System.out, "UTF-8");
    }

}